--[[################################################################################
    # File      : /hook/lua/ui/controls/DnDcombo.lua                               #
    # Author(s) : Ted Snook + Manimal                                              #
    # Version   : 20 mars 2009                                                     #
    # Revision  : 04 avril 2009                                                    #
    #                                                                              #
    # Summary   : New Combo Box with Drag and Drop Facility                        #
    # Copyright :  2006 Gas Powered Games, Inc. All rights reserved.              #
    ################################################################################]]--

# NBR LIGNE ORIGINAL = 674

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# NOTE :
# ======
# 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


################################################################################
# IMPORTS Perso

local Game = import( '/lua/game.lua' )
local MonMod = Game.MonMod
local MyModPath = Game.MyModPath
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


--[[
-- A combo box consists of a text control, a button to expand it, and a dropdown selectable list
-- this is a custom control as it its default has very game specific look to it
-- Combo box will need to have its width set, but height will be auto based on the bitmaps
]]--

################################################################################
# IMPORTS

local LazyVar = import('/lua/lazyvar.lua')
local UIUtil = import('/lua/ui/uiutil.lua')
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local Group = import('/lua/maui/group.lua').Group
local Text = import('/lua/maui/text.lua').Text
local ItemList = import('/lua/maui/itemlist.lua').ItemList
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local Dragger = import('/lua/maui/dragger.lua').Dragger
local UIMain = import('/lua/ui/uimain.lua')


################################################################################
# VARIABLES

local activeCombo = nil

local defaultBitmaps = {
    button = {
        left = {
            up = UIUtil.SkinnableFile('/widgets/drop-down/drop_btn_up_l.dds'),
            down = UIUtil.SkinnableFile('/widgets/drop-down/drop_btn_down_l.dds'),
            over = UIUtil.SkinnableFile('/widgets/drop-down/drop_btn_over_l.dds'),
            dis = UIUtil.SkinnableFile('/widgets/drop-down/drop_btn_dis_l.dds'),
        },
        mid = {
            up = UIUtil.SkinnableFile('/widgets/drop-down/drop_btn_up_m.dds'),
            down = UIUtil.SkinnableFile('/widgets/drop-down/drop_btn_down_m.dds'),
            over = UIUtil.SkinnableFile('/widgets/drop-down/drop_btn_over_m.dds'),
            dis = UIUtil.SkinnableFile('/widgets/drop-down/drop_btn_dis_m.dds'),
        },
        right = {
            up = UIUtil.SkinnableFile('/widgets/drop-down/drop_btn_up_r.dds'),
            down = UIUtil.SkinnableFile('/widgets/drop-down/drop_btn_down_r.dds'),
            over = UIUtil.SkinnableFile('/widgets/drop-down/drop_btn_over_r.dds'),
            dis = UIUtil.SkinnableFile('/widgets/drop-down/drop_btn_dis_r.dds'),
        }
    },
    list = {
       ul = UIUtil.SkinnableFile('/widgets/drop-down/drop-box_brd_ul.dds'),
       um = UIUtil.SkinnableFile('/widgets/drop-down/drop-box_brd_horz_um.dds'),
       ur = UIUtil.SkinnableFile('/widgets/drop-down/drop-box_brd_ur.dds'),
       l = UIUtil.SkinnableFile('/widgets/drop-down/drop-box_brd_vert_l.dds'),
       m = UIUtil.SkinnableFile('/widgets/drop-down/drop-box_brd_m.dds'),
       r = UIUtil.SkinnableFile('/widgets/drop-down/drop-box_brd_vert_r.dds'),
       ll = UIUtil.SkinnableFile('/widgets/drop-down/drop-box_brd_ll.dds'),
       lm = UIUtil.SkinnableFile('/widgets/drop-down/drop-box_brd_lm.dds'),
       lr = UIUtil.SkinnableFile('/widgets/drop-down/drop-box_brd_lr.dds'), 
    },
}


################################################################################
# VARIABLES Perso

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local MouseSelections = import( MyModPath .. '/hook/lua/ui/controls/MouseSelections.lua' )
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
local RaZselections = MouseSelections.RaZselections
local SingleSelectionUpdateList = MouseSelections.SingleSelectionUpdateList
local TogglableSelectionUpdateList = MouseSelections.TogglableSelectionUpdateList
local MultipleSelectionUpdateList = MouseSelections.MultipleSelectionUpdateList
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-- FOR SOME REASON, MY BITMAPS SHOW UP AT A REDUCED SIZE ! ( 225x13 instead of 227x15 )  DESPITE TEXTURE BORDER SETTING AT ZERO !!!  o_O
local itemSelectedTexture = UIUtil.UIFile( '/widgets/item_selected.dds' )
local itemDraggedTexture = UIUtil.UIFile( '/widgets/item_dragged.dds' )
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



--[[################################################################################
# FONCTIONS
################################################################################]]--


DnDcombo = Class(Group) {
    __init = function( self, parent, pointSize, maxVisibleItems, staticTitle, bitmaps, rolloverCue, clickCue, itemCue, debugName )
        Group.__init( self, parent )
        self:SetName( debugName or "DnDcombo" )
        self.mRolloverCue = rolloverCue
        self.mClickCue = clickCue
        self.mItemCue = itemCue or "UI_Tab_Click_01"

        bitmaps = bitmaps or defaultBitmaps

        pointSize = pointSize or 12
        maxVisibleItems = maxVisibleItems or 10

        -- this sets if the title is fixed or changes when a new selection is made
        self._staticTitle = staticTitle

        self._btnLeft = Bitmap(self, bitmaps.button.left.up)
        self._btnRight = Bitmap(self, bitmaps.button.right.up)
        self._btnMid = Bitmap(self, bitmaps.button.mid.up)
        self._btnLeft:DisableHitTest()
        self._btnRight:DisableHitTest()
        self._btnMid:DisableHitTest()
        
        LayoutHelpers.AtLeftIn(self._btnLeft, self)
        LayoutHelpers.AtTopIn(self._btnLeft, self)
        LayoutHelpers.AtRightIn(self._btnRight, self)
        LayoutHelpers.AtTopIn(self._btnRight, self)
        LayoutHelpers.AtTopIn(self._btnMid, self)
        self._btnMid.Left:Set(function() return self._btnLeft.Right() - 1 end)
        self._btnMid.Right:Set(self._btnRight.Left)

        self._text = UIUtil.CreateText(self._btnMid, "", pointSize, UIUtil.bodyFont)
        self._text:DisableHitTest()

        -- text control is height of text/font, and from left to button
        self.Height:Set(function() return math.max(self._text.Height(), self._btnMid.Height()) end)
        self._text.Top:Set(self._btnMid.Top)
        self._text.Left:Set(function() return self._btnLeft.Left() + 5 end)
        self._text.Right:Set(function() return self._btnMid.Right() + 5 end)
        self._text:SetClipToWidth(true)

        self._dropdown = Group(self._text)
        self._dropdown.Top:Set(self.Bottom)
        self._dropdown.Right:Set(function() return self.Right() end)
        self._dropdown.Width:Set(function() return self.Width() - 5 end)
    
        local ddul = Bitmap(self._dropdown, bitmaps.list.ul)
        local ddum = Bitmap(self._dropdown, bitmaps.list.um)
        local ddur = Bitmap(self._dropdown, bitmaps.list.ur)
        local ddl = Bitmap(self._dropdown, bitmaps.list.l)
        local ddm = Bitmap(self._dropdown, bitmaps.list.m)
        local ddr = Bitmap(self._dropdown, bitmaps.list.r)
        local ddll = Bitmap(self._dropdown, bitmaps.list.ll)
        local ddlm = Bitmap(self._dropdown, bitmaps.list.lm)
        local ddlr = Bitmap(self._dropdown, bitmaps.list.lr)

        -- top part is fixed under self        
        LayoutHelpers.AtLeftIn(ddul, self._dropdown)
        LayoutHelpers.AtTopIn(ddul, self._dropdown)
        LayoutHelpers.AtRightIn(ddur, self._dropdown)
        LayoutHelpers.AtTopIn(ddur, self._dropdown)
        LayoutHelpers.AtTopIn(ddum, self._dropdown)
        ddum.Left:Set(ddul.Right)
        ddum.Right:Set(ddur.Left)

        self._list = ItemList(ddm)   -- make list depth over text so if you have them stacked, you see list
        self._list:SetFont(UIUtil.bodyFont, pointSize)
        self._list:SetColors(UIUtil.fontColor, "Black", "Black", "Gainsboro")
        if staticTitle then
            self._list:ShowSelection(false)
        end
        self._list:ShowMouseoverItem(true)

        -- middle part is fixed to width, set to height of item list
        ddl.Top:Set(ddul.Bottom)
        ddl.Left:Set(self._dropdown.Left)
        ddr.Top:Set(ddur.Bottom)
        ddr.Right:Set(self._dropdown.Right)
        ddm.Top:Set(ddum.Bottom)
        ddm.Left:Set(ddl.Right)
        ddm.Right:Set(ddr.Left)
        ddm.Height:Set(self._list.Height)
        ddl.Height:Set(self._list.Height)
        ddr.Height:Set(self._list.Height)
        ddll.Bottom:Set(self._dropdown.Bottom)
        ddll.Left:Set(self._dropdown.Left)
        ddlr.Bottom:Set(self._dropdown.Bottom)
        ddlr.Right:Set(self._dropdown.Right)
        ddlm.Bottom:Set(self._dropdown.Bottom)
        ddlm.Left:Set(ddll.Right)
        ddlm.Right:Set(ddlr.Left)

        -- list is always under the control and the width (scrollbar under button)
        self._list.Top:Set(ddm.Top)
        self._list.Left:Set(ddm.Left)
        self._list.Right:Set(ddm.Right)
        
		--[[ #################################################################################
		       N E W   F E A T U R E  
		     #################################################################################]]--
		
		-- INIT SECTION
		self._list._isMoved = false
		self._list._isItemDropped = false
		self._list._currRow = 0
		self._list._srceRow = 0
		self._list._destRow = 0
		
		selection = {		-- Keep Track of Selections done by Mouse Clicks
			prem = -1, dern = -1,
			orig = -1, dest = -1,
			historik = { }, --{ 1, etc, n },
			items = { },    --{ 1, etc, n },
		}
		
		itemDragger = false
		itemBitmap = false
		
		
		function KillItemDragger()
			if itemDragger then
				itemDragger:Destroy()
				itemDragger = false
			end
		end
		
		function KillItemBitmap()
			if itemBitmap then
				itemBitmap:Destroy()
				itemBitmap = false
			end
		end
		
		
		-- ITEM BITMAP will represent the dragged item  -- row IS 1 BASED  BUT NUMBER OF ROWS IS ( n - 1 ) BASED 
		function CreateItemBitmap( row, itemTexture )
			itemBitmap = Bitmap( self._list )
			itemBitmap.Depth:Set( function() return self._list.Depth() + 1 end )
			local draggedText = self._list:GetItem( row - 1 ) -- want to read what is dragged
			-- set Font and Color of itemBitmap text relative to ItemList text ones
			--itemBitmap._text:SetColor( self._titleColor or UIUtil.fontColor ) -- Requires Faction Colors for the Background Texture ( Note from Manimal : I was too lazy for doing that XD)
			itemBitmap._text = UIUtil.CreateText( itemBitmap, draggedText, pointSize, UIUtil.bodyFont )
			itemBitmap._text:SetColor( "Black" ) -- Faction Agnostic Color for the Dragged Text
			-- set size of itemBitmap text relative to ItemList text
			itemBitmap:SetTexture( itemTexture, 0 )   -- TEXTURES ARE REDUCED DESPITE BORDER SETTING ! 
			itemBitmap.Height:Set( function() return (self._text.FontAscent() + self._text.FontDescent() + self._text.FontExternalLeading()) end )
			itemBitmap.Width:Set( function() return self._list.Width() end )
			itemBitmap.Top:Set( function() return self._list.Top() + ( ( row - 1 ) * itemBitmap.Height() ) end )
			itemBitmap.Left:Set( function() return self._list.Left() end )
			itemBitmap.Right:Set( function() return self._list.Right() end ) -- INDISPENSABLE ?
			LayoutHelpers.AtHorizontalCenterIn( itemBitmap, self._list )
			--- text position is relative to its container
			itemBitmap._text.Top:Set( function() return itemBitmap.Top() end  )
			itemBitmap._text.Left:Set( function() return itemBitmap.Left() end )
			itemBitmap._text.Right:Set( function() return self._list.Right() end ) -- INDISPENSABLE
			itemBitmap._text:SetCenteredVertically( true )
			itemBitmap._text:SetClipToWidth( true )
			itemBitmap._text:DisableHitTest()
		end
		
		
		--[[ 
		-- ATTENTION PERTE POSSIBLE DU FOCUS PAR  OnClick ! 
		-- DEVRAIT TENIR COMPTE DE event.Modifier.Left POUR EXTENSION 
		]]--
		self._list.OnMouseBtnPress = function( list, event )
			self._list._isMoved = false
			self._list._srceRow = self._list._currRow -- ATTENTION PERTE POSSIBLE DU FOCUS PAR OnMouseOverItem
			
			# > CREATION DE BITMAP DANS OnMove AMELIORERAIT GESTION DES CLICS ???
			KillItemDragger()      --  PAS DE DOUBLONS itemDragger
			KillItemBitmap()       --  PAS DE DOUBLONS itemBitmap
			if ( self._list._srceRow ~= -1  ) then
				-- THE DRAGGED ITEM AND ITS TEXT ATTACHED TO IT.
				itemDragger = Dragger()
				CreateItemBitmap( self._list._srceRow, itemDraggedTexture )
			end
			
			# DRAG DOIT VIVRE JUSQUE DROP
			# > PRVOIR MARGE DE DTECTION DE Move ? ( GENRE x+2 ET/OU y+2 )
			itemDragger.OnMove = function( list, x, y )  -- VOLE LE FOCUS DE OnMouseOverItem DONC IMPOSSIBLE DE RCUPRER row
				self._list._isMoved = true
				self._list._isItemDropped = false -- REINIT
				local newX = self:_ConstrainX( x )   -- Current Position of the item on axis X
				local newY = self:_ConstrainY( y )   -- Current Position of the item on axis Y
				itemBitmap.Left:Set( newX )  -- Afficher dans les limites
				itemBitmap.Top:Set( newY )   -- Afficher dans les limites
				local row = self:GetItemRowFromMousePos( newY )  -- NE RIEN FAIRE SI row = -1  <=>  LIGNE  MORTE  SOUS LE DERNIER ITEM
				if ( row > 0 ) and ( row <= self._visibleItems() ) then
					self._text:SetText( self._list:GetItem( row - 1 ) )  -- indique item survol
				end
				self:OnDragItem()
			end
			
			itemDragger.OnRelease = function( list, x, y )  -- VOLE LE FOCUS DE OnMouseOverItem DONC IMPOSSIBLE DE RCUPRER ROW
				local newX = self:_ConstrainX( x )   -- Current Position of the item on axis X
				local newY = self:_ConstrainY( y )   -- Current Position of the item on axis Y
				local row = self:GetItemRowFromMousePos( newY )  -- NE RIEN FAIRE SI row = -1  <=>  LIGNE  MORTE  SOUS LE DERNIER ITEM
				if ( row > 0 ) and ( row <= self._visibleItems() ) and ( self._list._srceRow ~= row ) then
					self._list._destRow = row  -- CONSTRAINED row WITHIN RANGE [ 1..visibleItems ]
					self:UpdateItemList( self._list._srceRow, self._list._destRow )
					self._list._isItemDropped = true
					self:OnDropItem()
				end
				KillItemBitmap()  # DROP = FIN DE << VIE >> DU BITMAP   -- NETTOYAGE..
				KillItemDragger() # DROP = FIN DE << VIE >> DU DRAGGER  -- NETTOYAGE..
			end
			
			itemDragger.OnCancel = function( list )
				KillItemBitmap()        -- NETTOYAGE...
				KillItemDragger()       -- NETTOYAGE...
			end
			
			if itemDragger then   -- itemDragger PAS CR SI row EST HORS LIMITES OU SI  row = -1 ! 
				PostDragger( self:GetRootFrame(), event.KeyCode, itemDragger )
			end
		end  -- FIN DU HANDLER OnMouseBtnPress
		
		
		
		-- event are handled there : 
		self._list.HandleEvent = function( control, event )
			if event.Type == 'MouseExit' then
				self:OnMouseExit()
 			elseif event.Type == 'ButtonPress' then  -- AJOUT DE NOUVEAU Handler
				self._list:OnMouseBtnPress( event )
			end
		end
		
		
        -- hide the list by default
        self._listhidden = true
        self._dropdown:Hide()
      
        -- supress show when list is in hidden state
        self._dropdown.OnHide = function(ctrl, hidden)
            if not hidden and self._listhidden then
                return true
            end

            if self:IsDisabled() then
                self._listhidden = true
                activeCombo = nil
                return
            end
            
            if not hidden then
                self._btnLeft:SetTexture(bitmaps.button.left.down)
                self._btnRight:SetTexture(bitmaps.button.right.down)
                self._btnMid:SetTexture(bitmaps.button.mid.down)
                self._text:SetColor("Black")
                self:OnMouseExit()
                if activeCombo and activeCombo != self then
                    activeCombo._listhidden = true
                    activeCombo._dropdown:SetHidden(true)
                end
                activeCombo = self
                self._dropdown.Depth:Set( self:GetRootFrame():GetTopmostDepth() + 1 )
            else
                self._btnLeft:SetTexture(bitmaps.button.left.up)
                self._btnRight:SetTexture(bitmaps.button.right.up)
                self._btnMid:SetTexture(bitmaps.button.mid.up)
                self._text:SetColor( self._titleColor or UIUtil.fontColor )
                activeCombo = nil
            end
        end
		
        -- set the height of the list based on the number of items visible and the font metrics
        self._maxVisibleItems = maxVisibleItems
        self._visibleItems = LazyVar.Create()
		-- Hauteur calculee a partir de Font  = 15 (si Font change alors Hauteur aussi )
        self._list.Height:Set(function() return self._visibleItems() * (self._text.FontAscent() + self._text.FontDescent() + self._text.FontExternalLeading()) end)
        self._dropdown.Height:Set(function() return self._list.Height() + ddum.Height() + ddlm.Height() end)
        self._visibleItems:Set(1)  # WHAT THAT ? WOW ! ONLY 1 VISIBLE ITEM OR ?
		
        -- SET UP BUTTON LOGIC
		-- ~~~~~~~~~~~~~~~~~~~
        self.HandleEvent = function( ctrl, event )
            local eventHandled = false

            if self:IsDisabled() then
                return eventHandled
            end
			
            if event.Type == 'MouseEnter' then
                if self._dropdown:IsHidden() then
                    self._btnLeft:SetTexture(bitmaps.button.left.over)
                    self._btnRight:SetTexture(bitmaps.button.right.over)
                    self._btnMid:SetTexture(bitmaps.button.mid.over)
                    self._text:SetColor(self._titleColor or UIUtil.fontColor)
                    if self.mRolloverCue then
                        local sound = Sound({Cue = self.mRolloverCue, Bank = "Interface",})
                        PlaySound(sound)
                    end
                end
                self:OnMouseEnter()  # Pour Affichage Tooltips de staticTitle notamment # 
                eventHandled = true
				
            elseif event.Type == 'MouseExit' then
                if self._dropdown:IsHidden() then
                    self._btnLeft:SetTexture(bitmaps.button.left.up)
                    self._btnRight:SetTexture(bitmaps.button.right.up)
                    self._btnMid:SetTexture(bitmaps.button.mid.up)
                    self._text:SetColor( self._titleColor or UIUtil.fontColor )
                end
                self:OnMouseExit()  # Pour Affichage Tooltips de itemList notamment # 
                eventHandled = true
			
            elseif event.Type == 'ButtonPress' then
				self._listhidden = not self._dropdown:IsHidden()
				self._dropdown:SetHidden( not self._dropdown:IsHidden() )
				if self.mClickCue then
					local sound = Sound( {Cue = self.mClickCue, Bank = "Interface",} )
					PlaySound( sound )
				end
				--eventHandled = true  # EMPECHE LA RECUPERATION DE EVENT ButtonPress ET ButtonClick ESSENTIELS POUR DragNdrop
            end
			
            self:OnEvent(event)
                        
            return eventHandled
        end
        
        self.OnDisable = function(self)
            self._btnLeft:SetTexture(bitmaps.button.left.dis)
            self._btnRight:SetTexture(bitmaps.button.right.dis)
            self._btnMid:SetTexture(bitmaps.button.mid.dis)
            self._text:SetColor(self._titleColor or UIUtil.fontColor)
            self._dropdown:Hide()
        end
        
        self.OnEnable = function(self)
            self._btnLeft:SetTexture(bitmaps.button.left.up)
            self._btnRight:SetTexture(bitmaps.button.right.up)
            self._btnMid:SetTexture(bitmaps.button.mid.up)
            self._text:SetColor(self._titleColor or UIUtil.fontColor)
        end
		
		
		
		-- SET UP SELECTION LOGIC
		-- ~~~~~~~~~~~~~~~~~~~~~~
		
		--[[
		# MULTI-SLECTION avec CTRL + LEFT CLICK  ou SHIFT + LEFT CLICK 
		]]--
		self._list.OnClick = function( list, row, event )
			if ( event.Modifiers.Left ) and not( self._list._isMoved ) and not( self._list._isItemDropped ) then
				self._list._currRow = row + 1
				SingleSelectionUpdateList( selection, self._list._currRow )
				ItemList.OnClick( self._list, row )
				if not( self._staticTitle ) then
					self._text:SetText( self._list:GetItem( row ) )
				end
				self:OnClick( row + 1, self._list:GetItem( row ) )
			end
		end
		
		
		self._list.OnMouseoverItem = function( list, row )
			if self.mItemCue then
				local sound = Sound( {Cue = self.mItemCue, Bank = "Interface",} )
				PlaySound( sound )
			end
			if row == -1 then
				self._list._currRow = -1
				self:OnOverItem( -1, nil )
			else
				self._list._currRow = row + 1
				self:OnOverItem( row + 1, self._list:GetItem( row ) )
			end
		end
		
		
		
		OnGlobalMouseClick = function( event )
			if self and self._list then
				if not self._listhidden then
					local rightCheck
					if self._scrollbar then
						rightCheck = self._scrollbar.Right()
					else
						rightCheck = self.Right()
					end
					if (event.x < self.Left() or event.x > rightCheck) or (event.y < self.Top() or event.y > self._dropdown.Bottom()) then
						self._listhidden = not self._dropdown:IsHidden()
						self._dropdown:SetHidden(not self._dropdown:IsHidden())
					end
				end
			end
		end
		
		UIMain.AddOnMouseClickedFunc(OnGlobalMouseClick)
		
		self.OnDestroy = function(self)
			UIMain.RemoveOnMouseClickedFunc(OnGlobalMouseClick)
		end
		
    end,
	

	-- Add items from a table of strings, also set the visible size
	-- defaultItemIndex is 1 based
    AddItems = function( self, textArray, defaultItemIndex )
        defaultItemIndex = defaultItemIndex or 1
        local numItems = table.getn( textArray )
        self._visibleItems:Set(  math.min( numItems, self._maxVisibleItems )  )

        if self._scrollbar then
            self._scrollbar:Destroy()
        end
        if numItems > self._visibleItems() then
            self._scrollbar = UIUtil.CreateVertScrollbarFor( self._list )
        end

        for i, text in ipairs( textArray ) do
            self._list:AddItem(  LOC( text )  )
        end

        self:SetItem( defaultItemIndex )
    end,
    
	-- remove old items, also set the visible size
    ClearItems = function( self )
        self._visibleItems:Set(0)
        self._list:DeleteAllItems()
        self._text:SetText("")
        if self._scrollbar then
            self._scrollbar:Destroy()
            self._scrollbar = nil
        end
    end,

    -- set the index selected (1 based!)
    SetItem = function( self, index )
        ItemList.OnClick( self._list, index - 1 )
        if not self._staticTitle then
			self._text:SetText(  self._list:GetItem( index - 1 )  )
		end
    end,

    -- get the index selected (1 based!) and the item
    GetItem = function( self )
        local sel = self._list:GetSelection()
        if sel >= 0 then
            return sel + 1, self._list:GetItem( sel )
        else
            return -1, nil
        end
    end,

    SetTitleText = function( self, text )
        self._text:SetText( text )
    end,
    
    SetTitleTextColor = function( self, color )
        self._titleColor = color
        self._text:SetColor( color )
    end,

    -- overload to get clicks
    OnClick = function( self, index, text )
    end,
    
    OnEvent = function( self )
    end,
    
    OnMouseEnter = function( self )
    end,
    
    OnMouseExit = function( self )
    end,
    
    -- overload to get rolled over item index
    OnOverItem = function( self, index, text )
    end,
	
	
	# ===========================================================================================================
	# N E W   M E T H O D S   B E L O W
	# ===========================================================================================================
	
	
	-- Get row from Mouse Coordinates  ( 1 BASED ! )
	GetItemRowFromMousePos = function( self, posY )
		local row = MATH_IRound(  ( posY - self._list.Top() )  /  itemBitmap.Height() ) + 1	-- CALCULER row  PARTIR DE event.MouseY
		return math.max(  math.min( row, self._visibleItems() ), 1  )  -- CONSTRAIN row WITHIN RANGE [ 1..visibleItems ]
	end,
	
	
	--# DPLACEMENT HORIZONTAL ALIGN sur BORD GAUCHE de la ZONE CIBLE (target) # DOIT tenir compte de la LARGEUR de la CIBLE
	_ConstrainX = function( self, posX )
		return self._list.Left()  -- X mouvant BORD GAUCHE dans le cadre ItemList
	end,
	
	--# DPLACEMENT VERTICAL ALIGN sur ITEM COURANT de la ZONE CIBLE ( ItemList )  # DOIT tenir compte de la HAUTEUR de la CIBLE ET INTERDIR LA ZONE  MORTE  DANS ItemList (row = -1 !)
	_ConstrainY = function( self, posY )
		local curRow = self:GetItemRowFromMousePos( posY ) - 1  --# GetItemRowFromMousePos IS 1 BASED !
		local fixedY = self._list.Top() + ( curRow * itemBitmap.Height() )
		local bottom = self._list.Top() + ( ( self._visibleItems() - 1 ) * itemBitmap.Height() ) --# coordinates are 0 based, so this will point to top of last row ( 1st pixel )
		return math.max(  math.min( fixedY, bottom ), self._list.Top()  )  --# y mouvant par "bonds" sur items dans le cadre target
	end,
	
	
	-- Get all the Items ( 1 based ! )
	GetAllItems = function( self )
		local itemsTbl = {}
		if self._list and not( self._listhidden ) then
			for row = 1, self._maxVisibleItems do
				table.insert( itemsTbl, row, self._list:GetItem( row - 1 )  )
			end
		end
		return itemsTbl
    end,
    
	-- defaultItemIndex is 1 based
	GetVisibleItems = function( self )
		local itemsTbl = {}
		if self._list and not( self._listhidden ) then
			for row = 1, self._visibleItems() do
				table.insert( itemsTbl, row, self._list:GetItem( row - 1 )  )
			end
		end
		return itemsTbl
    end,
    
	
	--[[  LA MTHODE CI-DESSOUS FONCTIONNE BIEN MAIS SEMBLE POSER DES PROBLMES DE SLECTION (  JAMAIS EFFECTIVE QUOIQUE JE FASSE )
	]]--
	--  ( IS 0 BASED )  -- INUTILE APRS ModifyItem ???
	UpdateItemSelection = function( self, row )
		self._list:SetSelection( row )  -- SLECTIONNE Item DPLAC
		if not( self._staticTitle ) then
			self._text:SetText( self._list:GetItem( row ) ) -- AFFICHE TEXTE Item DPLAC
		end
	end,
	
	
	-- SWAP srce ITEM AND dest ITEM  ( IS 1 BASED )
	SwapItems = function( self, srceRow, destRow )
		if self._list and not( self._listhidden ) then
			local visItems = self._visibleItems()
			if  ( srceRow > 0 ) and ( srceRow <= visItems ) and ( destRow > 0 ) and ( destRow <= visItems ) then
				local srceItem = self._list:GetItem( srceRow - 1 )
				local destItem = self._list:GetItem( destRow - 1 )
				self._list:ModifyItem( srceRow - 1, destItem )
				self._list:ModifyItem( destRow - 1, srceItem )
				self:UpdateItemSelection( destRow - 1 )  -- Note from Manimal: LAST MODIFIED ITEM IS NEVER SELECTED !!! -_-
			end
		end
	end,
    
	-- SHIFT ITEMS BETWEEN srce  AND dest  ( IS 1 BASED )
	ShiftItems = function( self, srceRow, destRow )
		local visItems = self._visibleItems()
		if ( destRow ~= srceRow ) and ( srceRow > 0 ) and ( srceRow <= visItems ) and ( destRow > 0 ) and ( destRow <= visItems ) then
			local deb = -1     -- INIT
			local fin = -1     -- INIT
			local positif = ( ( destRow - srceRow ) > 0 )  -- positif = true, negatif = false
			if  positif   then       -- DFINIR LA DIRECTION DU DCALAGE, POSITIF = MONTE
				deb = srceRow - 1
				fin = destRow - 1
			else                  -- DFINIR LA DIRECTION DU DCALAGE, NGATIF = DESCEND
				deb = destRow - 1
				fin = srceRow - 1
			end
			fin = fin - 1  -- ( n - 1 ) based ! 
			local shiftItem = self._list:GetItem( srceRow - 1 ) -- SAUVER Item  DCALER
			for row = deb, fin do     -- DCALER ITEMS DEPUIS srce JUSQUE dest
				if positif then       -- DFINIR LA DIRECTION DU DCALAGE, POSITIF = MONTE
					local i = row
					local item = self._list:GetItem( i + 1 )
					self._list:ModifyItem( i, item )
				else                  -- DFINIR LA DIRECTION DU DCALAGE, NGATIF = DESCEND
					local i = ( fin - row )
					local item = self._list:GetItem( i )
					self._list:ModifyItem( i + 1, item )
				end
			end
			self._list:ModifyItem( destRow - 1, shiftItem )  -- RESTAURER Item  DCALER
			self:UpdateItemSelection( destRow - 1 ) -- Note from Manimal: WTF ! LAST MODIFIED ITEM IS NEVER SELECTED !!! -_-
		end
	end,
    
	
	-- SHIFT ITEMS BETWEEN srce  AND dest  ( IS 1 BASED )
	UpdateItemList = function( self, srceRow, destRow )
		local visItems = self._visibleItems()
		if ( ( destRow - srceRow ) == 1 ) or ( ( srceRow - destRow ) == 1 ) then
			self:SwapItems( srceRow, destRow )
		else
			self:ShiftItems( srceRow, destRow )
		end
	end,
    
	
	-- overload to be informed when the item is droppable
	isItemDropped = function( self )
		if self._list and not( self._listhidden ) then
			return self._list._isItemDropped
		end
		return false
	end,
	
	-- overload to be informed when someone is dropping the item
	OnDragItem = function( self )
	end,
	
	-- overload to be informed when someone is dropping the item
	OnDropItem = function( self )
	end,
	
}

