--[[################################################################################
    # File      : /hook/lua/ui/controls/MouseSelections.lua                        #
    # Author(s) : Manimal                                                          #
    # Version   : 26 mars 2009                                                     #
    # Revision  : 04 avril 2009                                                    #
    #                                                                              #
    # Summary   : Mouse Selection Utilities for my New Combo Box with Drag & Drop  #
    # Copyright : CopyLeft 2009 Manimal.                                           #
    ################################################################################]]--

-- RaZselections( liste )
-- ExisteDansListe( liste, numItem )
-- SingleSelectionUpdateList( row )
-- TogglableSelectionUpdateList( row )
-- MultipleSelectionUpdateList( row )

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# NOTE :
# ======
# 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

--[[################################################################################
	liste = {		-- Keep Track of Selections done by Mouse Clicks	# TABLE PAR DEFAUT self._list.selected
		prem = -1,         -- DEFAULT VALUE IS << EMPTY >> LIST THAT IS NO prem
		dern = -1,         -- DEFAULT VALUE IS << EMPTY >> LIST THAT IS NO dern
		orig = -1,         -- DEFAULT VALUE IS << NOWHERE >> THAT IS NO origin
		dest = -1,         -- DEFAULT VALUE IS << NOWHERE >> THAT IS NO destination
		historik = {},     -- { 1, etc, n },
		items = {},        -- { 1, etc, n    }   -- ANCIENNE VERSION {  { row = -1, bmp = false },  }
	}
    ################################################################################]]--

################################################################################
# << CONSTANTES >>

local VIDE = -1
local MINI = 1
local MAXI = 7777777


################################################################################
# FONCTIONS


# PREMIER et DERNIER
# ==================

EstPremierSelection = function( liste, numItem )
	return numItem and ( numItem == liste.prem )    # QUID WHEN ( numItem ~= VIDE )
end


EstDernierSelection = function( liste, numItem )
	return numItem and ( numItem == liste.dern )    # QUID WHEN ( numItem ~= VIDE )
end



PeutRemplacerPremier = function( liste, numItem )
	return numItem and ( ( liste.prem == VIDE ) or ( numItem < liste.prem ) )
end


PeutRemplacerDernier = function( liste, numItem )
	return numItem and ( ( liste.dern == VIDE ) or ( numItem > liste.dern ) )
end



AddSelectedItemFirstToList = function( liste, numItem )
	liste.prem = numItem                         # INSERER TABLE : prem = numItem
end


RemoveSelectedItemFirstFromList = function( liste )
	liste.prem = VIDE                            # Rinit prem
	local plusPetit = MAXI
	for k, item in ipairs( liste.items ) do      # CHERCHER NOUVEAU prem :
		if PeutRemplacerPremier( liste, item ) and ( item < plusPetit ) then # SI LMENT COURANT PeutRemplacerPremier
			liste.prem = item                    # prem = PLUS PETIT LMENT TROUV
			plusPetit = item
		end
	end
end



AddSelectedItemLastToList = function( liste, numItem )
	liste.dern = numItem
end


RemoveSelectedItemLastFromList = function( liste )
	liste.dern = VIDE                            # Rinit dern
	local plusGrand = MINI
	for k, item in ipairs( liste.items ) do      # CHERCHER NOUVEAU dern :
		if PeutRemplacerDernier( liste, item ) and ( item > plusGrand ) then # SI LMENT COURANT PeutRemplacerDernier
			liste.dern = item                    # dern = PLUS GRAND LMENT TROUV
			plusGrand = item
		end
	end
end



# ORIGINE et DESTINATION
# ======================

ActualiserOrigine = function( liste, numItem )
	liste.orig = numItem                         # NOTER ITEM COURANT COMME NOUVELLE origine
	table.insert( liste.historik, numItem )      # NOTER ITEM COURANT DANS historik DES CHANGEMENTS D'ORIGINE
end


ReinitialiserOrigine = function( liste )
	liste.orig = VIDE
	liste.historik = {}
end


DefinirNouvelleOrigine = function( liste, numItem )
	if ( liste.orig == VIDE ) then               # SEULEMENT LORSQUE origine EST INDFINIE
		ActualiserOrigine( liste, numItem )		 # NOTER LMENT COURANT COMME origine
	end
end


RestaurerOriginePrecedente = function( liste )
	local lastItem = table.getn( liste.historik )
	if ( lastItem > 0 ) then
		liste.orig = liste.historik[lastItem]    # RCUPRER DERNIER ITEM COMME NOUVELLE origine
		table.remove( liste.historik )           # SUPPRIMER LE DERNIER LMENT DE historik
	end
end


DefinirNouvelleDestination = function( liste, numItem )
	if ( liste.orig > 0 ) then                   # SEULEMENT LORSQUE origine EST DJ DFINIE
		liste.dest = numItem                     # NOTER LMENT COURANT COMME destination
	end
end



# LMENTS
# ========

ExisteDansListe = function( liste, numItem )
	for k, item in ipairs( liste.items ) do
		if ( numItem == item ) then
			return true 
		end
	end
	return false  # RETOURNER INDEX TABLE ?
end



AddSelectedItemRowToList = function( liste, numItem )
	table.insert( liste.items, numItem )  # INSERER TABLE : row = numItem <=> INDEX de TABLE POUR SYNCHRO
end


RemoveSelectedItemRowFromList = function( liste, numItem )
	if liste.items then
		for k, item in ipairs( liste.items ) do
			if ( item  == numItem ) then
				table.remove( liste.items, k )       # SUPPRIMER item DANS TABLE 
				break
			end
		end
	end
end



# SLECTIONS
# ==========

AddCurrentSelectionToList = function( liste, numItem )
	if PeutRemplacerPremier( liste, numItem ) then   # CET LMENT PeutRemplacerPremier
		AddSelectedItemFirstToList( liste, numItem ) # NOTER CE CLIC COMME dbut DE SLECTION
	end
	if PeutRemplacerDernier( liste, numItem ) then   # CET LMENT PeutRemplacerDernier
		AddSelectedItemLastToList( liste, numItem )  # NOTER CE CLIC COMME fin DE SLECTION
	end
	AddSelectedItemRowToList( liste, numItem )       # AJOUT DIRECT
end


RemoveCurrentSelectionFromList = function( liste, numItem )
	RemoveSelectedItemRowFromList( liste, numItem )  # SUPPRESSION DIRECTE
	if EstPremierSelection( liste, numItem ) then    # CET LMENT EstPremierSelection
		RemoveSelectedItemFirstFromList( liste )     # SUPPRIMER CE CLIC COMME dbut DE SLECTION
	end
	if EstDernierSelection( liste, numItem ) then    # CET LMENT EstDernierSelection
		RemoveSelectedItemLastFromList( liste )      # SUPPRIMER CE CLIC COMME fin DE SLECTION
	end
end



RaZselections = function( liste )
	if liste then
		liste.prem = VIDE
		liste.dern = VIDE
		liste.orig = VIDE
		liste.dest = VIDE
		liste.items = {}
	end
end



# ==============================
# SIMPLE CLIC = 1 ITEM SEULEMENT
# ==============================

SingleSelectionUpdateList = function( liste, numItem )
	RaZselections( liste )                       # PURGE NOUVELLE VERSION ...
	ReinitialiserOrigine( liste )
	DefinirNouvelleOrigine( liste, numItem )     # DERNIRE SLECTION SERA TOUJOURS origine
	AddCurrentSelectionToList( liste, numItem )
end


# ===============================
# CTRL + CLIC = +1 ITEM  LA FOIS
# ===============================

TogglableSelectionUpdateList = function( liste, numItem )
	if ExisteDansListe( liste, numItem ) then    # CET LMENT EXISTE DANS LA TABLE DE SLECTION
		RestaurerOriginePrecedente( liste )
		RemoveCurrentSelectionFromList( liste, numItem )
	else
		DefinirNouvelleOrigine( liste, numItem ) # PREMIRE SLECTION SERA TOUJOURS origine JUSQUE SON EFFACEMENT
		AddCurrentSelectionToList( liste, numItem )
	end
end


# =================================
# SHIFT + CLIC = +n ITEMS  LA FOIS
# =================================

MultipleSelectionUpdateList = function( liste, numItem )
	DefinirNouvelleOrigine( liste, numItem )
	DefinirNouvelleDestination( liste, numItem ) # ORIGINE NE CHANGE PAS SYSTEMATIQUEMENT => AFFECTER SOUS CONDITION ANCIENNE VALEUR DE dest
	local debut = VIDE
	local fin = VIDE
	if ( liste.orig < liste.dest ) then
		debut = liste.orig                       # RESTAURER dbut DE SLECTION PRCDENT
		fin = liste.dest                         # RESTAURER fin DE SLECTION PRCDENTE
	elseif ( liste.orig == liste.dest ) then
		debut = liste.orig                       # RESTAURER dbut DE SLECTION PRCDENT
		fin = liste.dest                         # RESTAURER fin DE SLECTION PRCDENTE
	elseif ( liste.orig > liste.dest ) then
		debut = liste.dest                       # RESTAURER dbut DE SLECTION PRCDENT
		fin = liste.orig                         # RESTAURER fin DE SLECTION PRCDENTE
	end
	if ( debut > 0 ) and ( fin > 0 ) then
		liste.prem = VIDE
		liste.dern = VIDE
		liste.items = {}
		for item = debut, fin do                 # POUR TOUS LMENTS DANS LISTE FAIRE
			if not( ExisteDansListe( liste, item ) ) then
				AddCurrentSelectionToList( liste, item )   # AJOUTER LES ITEMS MANQUANTS DANS SELECTION
			end
		end
	end
end

