--[[################################################################################
    # File      : /hook/lua/ui/dialogs/ConsolePlusUtils.lua                        #
    # Author    : Manimal                                                          #
    # Version   : 25 septembre 2008                                                #
    # Revision  : 22 mars 2009                                                     #
    #                                                                              #
    # Summary   : Utility functions for Console Plus                               #
    # Copyright : CopyLeft 2009 Manimal.                                           #
    ################################################################################]]--

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# NOTE :
# ======
# ABANDON de TRI TABLE VIA TABLE d'INDEXES ?
# Mme si cette technique est -a priori- plus rapide en vitant de manipuler de longues
# chaines celle-ci peut causer une confusion entre table d'indexes et table de donnes.
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# NOTE :
# ======
# > > > CONTROLER INTERET DU break LIGNE 181 ET LIGNE 295 !!!
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


local Game = import('/lua/game.lua')
local MyModPath = Game.MyModPath

local Prefs = import( '/lua/user/prefs.lua' )


--[[################################################################################
# Manimal's Utility: REMOVES JUNK CHARACTERS from CORRUPTED LOCALIZED STRINGS
# THIS IS A TEMPORARY SOLUTION UNTIL GPG REPAIR THE LOC FILES !
################################################################################]]--

function FilteredLOC( dirtyString )
	if not dirtyString or not ( type(dirtyString) == 'string' ) then
		WARN("MANIMAL DEBUG: console.lua -> Error in FilteredLOC: dirtyString is missing or is not a string !")
		return false
	end
	
	dirtyString = LOC(dirtyString) # Get the original LOC string "as is".
	local myString = ""
	while true do
		local i, j = string.find( dirtyString, " " )
		if (i == nil) and (j == nil) then
			myString = dirtyString
			break
		end
		dirtyString = string.sub(dirtyString, 1, (i-1)) .. string.sub(dirtyString, (j+1))  # Remove the crap out of the string
	end

	return myString or false	# LUA REF does NOT tell what error could be returned by string.format
end



--[[################################################################################
# Manimal's Utility: REMOVES JUNK CHARACTERS from CORRUPTED LOCALIZED STRINGS
################################################################################
# AN ERROR CAN OCCURS WITH Prefs.GetFromCurrentProfile('console_alpha') ALONE BECAUSE IT IS SET FROM
# A MISSING FIELD WITHIN Game.Prefs, DUE TO THE LOCF ERROR BELOW THAT PREVENTS CONFIG TO UPDATE IT !!!
# ORIGINAL LINES ARE DISABLED BECAUSE LOCF RAISES A FORMAT ERROR DUE TO CRAP WITHIN LOCALIZED STRINGS
# -> local label = UIUtil.CreateText(client, LOCF("<LOC console_alpha>Alpha: %d%%", defValue), 14)
# -> label:SetText(LOCF("<LOC console_alpha>Alpha: %d%%", newValue))
# THIS IS A TEMPORARY SOLUTION UNTIL GPG REPAIR THE LOC FILES !
# -> label = UIUtil.CreateText( client, FilteredLOCF("<LOC console_alpha>Alpha: %d%%" , defValue), 14 )
# -> label:SetText( FilteredLOCF("<LOC console_alpha>Alpha: %d%%", newValue) )
################################################################################]]--

function FilteredLOCF( dirtyString, value )
	if not dirtyString or not ( type(dirtyString) == 'string' ) then
		WARN("MANIMAL DEBUG: console.lua -> Error in FilteredLOCF: dirtyString is missing or is not a string !")
		return false
	end
	if not value or not ( type(value) == 'number' ) then 
		WARN("MANIMAL DEBUG: console.lua -> Error in FilteredLOCF: value is missing or is not a number !")
		return false
	end
	
	dirtyString = LOC(dirtyString) # Get the original LOC string "as is".
	local myStringF = ""
	while true do
		local i, j = string.find( dirtyString, " " )
		if (i == nil) and (j == nil) then
			myStringF = dirtyString
			break
		end
		dirtyString = string.sub(dirtyString, 1, (i-1)) .. string.sub(dirtyString, (j+1))  # Remove the crap out of the string
	end
	myStringF = string.format( myStringF, value ) # Format the string representation of the value

	return myStringF or false	# LUA REF does NOT tell what error could be returned by string.format
end



--[[################################################################################
--# COMPARAISON CLASSIQUE DE NOMBRES - NUMBER COMPARATOR
--# RENVOIE nbA,nbB SI ERREUR OU UCUN NOMBRE; SINON '<', '=', '>'.
--################################################################################ ]]--

function Comparaison( valA, valB )
	if not(valA) or (valA == nil) then
		WARN('MANIMAL DEBUG:  Error in Comparaison: bad or missing parameter (valA) !' )
		return valA
	end
	if not(valB) or (valB == nil) then
		WARN('MANIMAL DEBUG:  Error in Comparaison: bad or missing parameter (valB) !' )
		return valB
	end
	
	local resultat = '?'
	
	if  type(valA) ~= type(valB)  then
		WARN('MANIMAL DEBUG:  Error in Comparaison: can not compare two different types (valA is a '..type(valA)..' and valB is a '..type(valB)..') !' )
		return resultat
	end
	-- From this point, TYPES should be EQUAL so we don't need to test BOTH value types again
	
	if  type(valA) == 'string'  then --# NON CASE SENSITIVE !!!
		if ( string.len( valA ) <= 0 ) or ( string.len( valB ) <= 0 ) then
			return resultat
		end
	
		valA = string.lower( valA )  
		valB = string.lower( valB )
	end
	
	if valA == valB then
		resultat = '='
	end
	if valA < valB then
		resultat = '<'
	end
	if valA > valB then
		resultat = '>'
	end
	
	return resultat
end


--[[################################################################################
# FONCTION DE TRI de TABLE AVEC CLEFS - Table Sorting utility BY KEY
################################################################################ ]]--

function TriTableAvecClefs( tabl, ordreTri, clefTri )
	if not tabl or (tabl == nil) or (type(tabl) ~= 'table') then
		WARN( '### MANIMAL DEBUG:  ConsolePlus.lua -> Error in TriTableAvecClefs: a table was expected  ! ###' )
		return tabl
	end
	if not(ordreTri) or (ordreTri == nil) or (type( ordreTri ) ~= 'string' ) or ( (ordreTri  ~= '<') and (ordreTri ~= '>') )  then
		WARN( '### MANIMAL DEBUG:  ConsolePlus.lua -> Error in TriTableAvecClefs: bad or missing sort operator, \"<\" or \">\" was expected !' )		--WARN( 'MANIMAL DEBUG:  Error in TriTableViaTableIndex:  bad or missing comparaison operator !' )
		return tabl
	end
	if not(clefTri) or (clefTri == nil) or ( ( type( clefTri ) ~= 'string' ) and ( type( clefTri ) ~= 'number' ) ) then
		WARN( '### MANIMAL DEBUG:  ConsolePlus.lua -> Error in TriTableAvecClefs: clef de tri non valide  ! ###' )
		return tabl
	end
	
	local TriAvecClef = (clefTri and (clefTri ~= nil) and ((type(clefTri) == 'string') or (type(clefTri) == 'number'))) or false 	--# Tri Table Index AVEC ou SANS CLEF
	local tableTriee = tabl    # Table de Travail, pour prserver tabl originale
	local nbrPermutations = 0
	
	repeat		--# TRIER TABLE JUSQUE ORDRE CROISSANT OU DCROISSANT	--#  ordreTri a t test plus haut et devrait tre OK
		nbrPermutations = 0

		for i, _ in pairs( tableTriee ) do	--# EXPLORER TABLE avec Indexation via CLEFS  (TOUS TYPES de CLEFS)
			if ( type( tableTriee[i][clefTri] ) == 'string' ) or ( type( tableTriee[i][clefTri] ) == 'number' ) then
				local j = i + 1
				if ( j ~= nil ) and ( tableTriee[j] ~= nil ) then
					local test = Comparaison( tableTriee[j][clefTri], tableTriee[i][clefTri] )
					if ( test == ordreTri )  then
						local element = tableTriee[i]
						tableTriee[i] = tableTriee[j]
						tableTriee[j] = element
						nbrPermutations = nbrPermutations + 1    # IL RESTE ENCORE LMENT  TRIER !
					end
				end
			--else  # NI CHAINE NI TABLE !
			--	break
			end
		end    -- end of FOR
		
	until ( nbrPermutations == 0 )
	
	return tableTriee
end


--[[################################################################################
--# FONCTION DE TRI de TABLE d'INDEXES - Table of Indexes Sorting Utility
--# Les indexes seront tries par rapport au contenu de 'tabl'
--################################################################################ ]]--

--[[--======================================================================================================
--# Initialisation de Table d'Indexes (de TOUS TYPES)
--====================================================================================================== ]]--
local function tIdxInit( t )
	local tIndx = {}
	for k, elTbl in pairs( t ) do	-- Init table indexes
		if type( elTbl ) == 'table' then
			table.insert( tIndx, k, tIdxInit( elTbl ) )		-- INDEXER TABLES
		else
			table.insert( tIndx, k )		-- INDEXER TOUT SAUF TABLES
		end
	end
	return tIndx
end


--[[--======================================================================================================
--# Tri capable de grer les clefs de type nombre et chaine
--# Si clefTri est spcifi alors TRI Valeur PAR CLEF, SINON TRI des Valeurs de TOUTES CLEFS !
--====================================================================================================== ]]--
function TriTableViaTableIndex( t, ordreTri, clefTri )
	if  not t or (t == nil) or ( type(t) ~= 'table' )  then
		WARN( 'Error in TriTableViaTableIndex: a table was expected !' )		--WARN( 'MANIMAL DEBUG:  Error in TriTableViaTableIndex: a table was expected  !' )
		return t
	end
	if  not(ordreTri) or (ordreTri == nil) or (type( ordreTri ) ~= 'string' ) or ( (ordreTri  ~= '<') and (ordreTri ~= '>') )  then
		WARN( 'Error in TriTableViaTableIndex: bad or missing sort operator, \"<\" or \">\" was expected !' )		--WARN( 'MANIMAL DEBUG:  Error in TriTableViaTableIndex:  bad or missing comparaison operator !' )
		return t
	end
		
	local TriAvecClef = (clefTri and (clefTri ~= nil) and ((type(clefTri) == 'string') or (type(clefTri) == 'number'))) or false 	--# Tri Table Index AVEC ou SANS CLEF
	local tIdx = tIdxInit( t )
	local nbrPermutations = 0
	
	repeat		--# TRIER 1 TABLE
		nbrPermutations = 0

		for i, iTi in pairs( tIdx ) do	--# EXPLORER TABLE avec Indexation par ORDRE ALPHABETIK de CLEFS  (TOUS TYPES)

			if (type( t[iTi] ) == 'table') then	--# A FAIRE ? -> NOTER le NIVEAU d'IMBRICATION de TABLE
				table.insert( tIdx, i, TriTableViaTableIndex( t[iTi], ordreTri, clefTri )  ) --# RCURSION !!!
				
			else
				local j = i + 1
				local jTi = tIdx[j]  --# INDEX SUIVANT trouv dans TABLE INDEXES
				if ( jTi ~= nil ) and ( t[jTi] ~= nil ) then
					local test = Comparaison( t[jTi], t[iTi] )
					if ( test == ordreTri )  then
						local preIdx = tIdx[i]
						tIdx[i] = tIdx[j]
						tIdx[j] = preIdx
						nbrPermutations = nbrPermutations + 1    --# IL RESTE ENCORE LMENT  TRIER !
					end
				end
			end
			
		end    -- end of FOR
		
	until ( nbrPermutations == 0 )
	
	return tIdx
end



--[[################################################################################
# FONCTION DE TRI de TABLE de CHAINES via INDEXES
# Table of Strings via INDEXES Sorting utility
################################################################################ ]]--

function TriTable( tabl, ordreTri, clefTri )
	if  not tabl or (tabl == nil) or (type(tabl) ~= 'table')  then
		WARN( 'Error in TriTable: a table was expected  !' )		--WARN( 'MANIMAL DEBUG:  Error in TriTable: a table was expected  !' )
		return tabl
	end
	
	local tIndex = TriTableViaTableIndex( tabl, ordreTri, clefTri )
	
	if  not tIndex or (tIndex == nil) or ( type( tIndex ) ~= 'table' ) then
		WARN( 'Error in TriTable: a table of indexes was expected  !' )		--WARN( 'MANIMAL DEBUG:  Error in TriTable: a table of indexes was expected  !' )
		return tabl
	end
	
	local tableTriee = {}  --# Table de Travail, pour prserver tabl originale
	
	for i, index in pairs( tIndex ) do
		if  type( index ) == 'table'  then
			table.insert( tableTriee, i, TriTable( tabl[i], ordreTri, clefTri )  )  --# RCURSION !!!
			
		elseif ( type( index ) == 'number' ) then
			tableTriee[i] = tabl[index]
			
		elseif ( type( index ) == 'string' ) then
			tableTriee[index] = tabl[index]	--rawset( tableTriee, index, tabl[index] ) fonctionne aussi bien !!!
			
		else  --# index est NI CHAINE NI NOMBRE NI TABLE !
			WARN( "\a\aTriTable -> BREAK !  (i = "..tostring(i)..") (type(index) = "..type(index)..")\n" )
			--break    -- Conserver pour cration Tri Autres  ?
		end
	end
	
	return tableTriee
end



--[[################################################################################
# FONCTION DE SAUVEGARDE et MISE  JOUR du BATCH dans GAME PREFS
# NOTE IMPORTANTE :
# IL SEMBLE QUE LA SAUVEGARDE du BATCH PERTURBE CERTAINES PREFERENCES AU POINT D EMPECHER LEUR PROPRE SAUVEGARDE
# L UTILISATION DE SavePreferences() PENDANT QUE LE MOTEUR UTILISE ET / OU MODIFIE CES PREFERENCES CAUSERAIT  LEUR PERTE.
# Batch Save and Update in GAME PREFS utility
# IMPORTANT NOTE :
# DO NOT INVOKE THIS FUNCTION BECAUSE IT DOES NOT WORK PROPERLY. (DATA SYNC TROUBLES WOULD OCCUR)
################################################################################ ]]--

function SaveBatchToPreferences( batch )
	local result = false
	if not( batch ) or ( batch == nil ) or ( type( batch ) ~= 'table' ) then
		WARN( "MANIMAL: Error SaveBatchToPreferences -> Wrong Type or Missing Batch ! (a table was expected)" )
		return result
	end
	
	local batchFieldName = 'ConsolePlus_command_batch'
	
	# Getting the Profiles
	local profile = GetPreference( 'profile' )
	local profiles = profile.profiles or false
	local current = profile.current or false
		
	# UPDATE BATCH DATA ( by processing CURRENT profile ONLY )
	# ==========================================================
    if profiles and current then
		local newProf = {}
		local change = false
		for kCur, vCur in pairs( profiles[current] ) do
			newProf[kCur] = vCur                          # Attribuer la Valeur Originale  la Clef Correspondante
			if ( kCur == batchFieldName ) then
				LOG( "MANIMAL: Saving Batch in Game Prefs under field name = "..tostring(kCur) )
				newProf[kCur] = batch                     # Attribuer la Nouvelle Valeur  la Clef Correspondante
				change = true
				break
			end
		end
		if change then
			table.remove( profiles, current )             # Suppression du Profil Courant Obsolete
			table.insert( profiles, current, newProf )    # Insertion du Profil Courant Mis  Jour
			SetPreference( 'profile.profiles', profiles ) # Validation des Changements dans la liste des Profils
			SavePreferences()                             # Sauvegarde des Changements dans Game.Prefs
			LOG( "MANIMAL: Batch Related Game Prefs Updates Saved To Disk !" )
		end
	end
end



--[[################################################################################
# FONCTION DE NETTOYAGE et MISE  JOUR de GAME PREFS
# REMARQUES : 
# CETTE FONCTION NE MODIFIE RIEN SI AUCUNE CLEF OBSOLETE TROUVE
# CETTE FONCTION NE MODIFIE PAS DES DONNES EN COURS D UTILISATION
# LES LISTES obsKeys ET newKeys SONT SYNCHRONISES ( ancienne clef de la premire table pointe la nouvelle clef dans la seconde table )
# GAME PREFS CLEANSING and UPDATING UTILITY IF NEEDED
# REMARKS:
# THIS FUNCTION DOES ALTER NOTHING IF NO OBSOLETE KEYS ARE FOUND
# THIS FUNCTION DOES NOT ALTER DATA THAT ARE IN USE YET
# obsKeys AND newKeys LISTS ARE SYNCHRONIZED ( old key of the first table pointing to the new key of the second table )
################################################################################ ]]--
	--[[
	profile = {
		current = 1,
		profiles = {
			{ Table des donnes du Profil 1 },
			{ Table des donnes du Profil 2 },
		},
	}
	]]--

function CleanUpGamePrefs()
	# Liste des Clefs Obsoletes dans Game.Prefs - List of Obsolete Keys within Game.Prefs
	# ===================================================================================
	local obsKeys = {
		'conplus_help_window',    # Nouveau nom = 'ConsolePlus_help_window'
		'console_alpha_locked',   # Nouveau nom = 'ConsolePlus_alpha_locked'
		'console_command_batch',  # Nouveau nom = 'ConsolePlus_command_batch'
		'cam_ShakeMult',          # Nouveau nom = 'ConsolePlus_cam_ShakeMult'
		'No_Shake',               # Nouveau nom = 'ConsolePlus_No_Shake'
	}
	# Liste des Nouvelles Clefs pour Game.Prefs - List of New Keys for Game.Prefs
	# ===========================================================================
	local newKeys = {
		'ConsolePlus_help_window',
		'ConsolePlus_alpha_locked',
		'ConsolePlus_command_batch',
		'ConsolePlus_cam_ShakeMult',
		'ConsolePlus_No_Shake',
	}
	# Getting the Profiles
	local profile = GetPreference( 'profile' )
	local profiles = profile.profiles or false
	local current = profile.current or false
		
	# UPDATE PROFILE DATA ( by processing CURRENT profile ONLY )
	# ==========================================================
    if profiles and current then
		local newProf = {}
		local change = false
		for kCur, vCur in pairs( profiles[current] ) do
			local fieldName = kCur                          # Rcupration de la Clef Originale
			for kObs, vObs in pairs( obsKeys ) do
				if ( vObs == kCur ) then
					fieldName = newKeys[kObs]               # Rcupration de la Nouvelle Clef
					LOG( "MANIMAL: Found Obsolete Game Prefs Key = "..tostring( kCur )..", Updating with New Key = "..tostring( fieldName ) )
					change = true
					break
				end
			end
			newProf[fieldName] = vCur                       # Attribuer la Valeur Originale  la Nouvelles Clef
		end
		if change then
			table.remove( profiles, current )               # Suppression du Profil Courant Obsolete
			table.insert( profiles, current, newProf )      # Insertion du Profil Courant Mis  Jour
			SetPreference( 'profile.profiles', profiles )   # Validation des Changements dans la liste des Profils
			SavePreferences()                               # Sauvegarde des Changements dans Game.Prefs
			LOG( "MANIMAL: Console Plus Related Game Prefs Updates Saved To Disk !" )
		end
	end
end

