--[[################################################################################
    # File      : /hook/lua/ui/game/TooltipPlus.lua                                #
    # Author    : Ted Snook + Manimal                                              #
    # Version   : 22 mars 2009                                                     #
    # Revision  : 22 mars 2009                                                     #
    #                                                                              #
    # Summary   : Tool Tips Plus for Console Plus v2                               #
    # Copyright :  2005 Gas Powered Games, Inc.  All rights reserved.             #
    ################################################################################]]--

# NBR LIGNES ORIGINAL = 331

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# NOTE :
# ======
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


local Game = import('/lua/game.lua')
local MyModPath = Game.MyModPath


################################################################################
# IMPORTS Original

local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local TooltipInfo = import('/lua/ui/help/tooltips.lua')
local Tooltip = import( '/lua/ui/game/tooltip.lua' )
local Prefs = import('/lua/user/prefs.lua')
local Keymapping = import('/lua/keymap/defaultKeyMap.lua').defaultKeyMap
local KeyBindings = import('/lua/ui/dialogs/keybindings.lua')


################################################################################
# VARIABLES Original

local mouseoverDisplay = false
local createThread = false


################################################################################
# IMPORTS Perso

# Liste obtenue par GetCommandsListFromConsole()
local CommandsList = import( MyModPath .. '/hook/lua/ui/dialogs/ConsoleCommands.lua').CommandsList


################################################################################
# VARIABLES Perso

TooltipInfoPlus = {}  # GLOBALE pour accs externe -- ALTERNATE Tooltip Infos (dynamic ones)


--[[################################################################################
# FONCTIONS
################################################################################]]--



function CreateMouseoverDisplay(parent, ID, delay, extendedBool, hotkeyID)
    if mouseoverDisplay then
        mouseoverDisplay:Destroy()
        mouseoverDisplay = false
    end
        
    if not Prefs.GetOption( 'tooltips' ) then return end
    local createDelay = 0
    if delay and Prefs.GetOption( 'tooltip_delay' ) then
        createDelay = math.max(  delay, Prefs.GetOption( 'tooltip_delay' )  )
    else
        createDelay = Prefs.GetOption( 'tooltip_delay' ) or 0
    end
    local totalTime = 0
    local alpha = 0.0
    local text = ""
    local body = ""
    if type(ID) == 'string' then
        if TooltipInfo['Tooltips'][ID] then
            text = LOC(TooltipInfo['Tooltips'][ID]['title'])
            body = LOC(TooltipInfo['Tooltips'][ID]['description'])
            if TooltipInfo['Tooltips'][ID]['keyID'] and TooltipInfo['Tooltips'][ID]['keyID'] != "" then
                for i, v in Keymapping do
                    if v == TooltipInfo['Tooltips'][ID]['keyID'] then
                        local properkeyname = KeyBindings.formatkeyname( i )
                        text = LOCF( "%s (%s)", text, properkeyname )
                        break
                    end
                end
            end
        elseif TooltipInfoPlus[ID] then
            text = LOC( TooltipInfoPlus[ID]['title'] )
            body = LOC( TooltipInfoPlus[ID]['description'] )
            if TooltipInfoPlus[ID]['keyID'] and TooltipInfoPlus[ID]['keyID'] != "" then
                for i, v in Keymapping do
                    if v == TooltipInfoPlus[ID]['keyID'] then
                        local properkeyname = KeyBindings.formatkeyname( i )
                        text = LOCF( "%s (%s)", text, properkeyname )
                        break
                    end
                end
            end
        else
            if extendedBool then
                WARN( "MANIMAL\`s TooltipPlus: No tooltip in table for key: "..ID )
            end
            text = ID
            body = "No Description"
        end
    elseif type(ID) == 'table' then
        text = LOC(ID.text)
        body = LOC(ID.body)
    else
        WARN( 'MANIMAL\`s TooltipPlus: UNRECOGNIZED TOOLTIP ENTRY - Not a string or table! ', repr(ID) )
    end
    if extendedBool then
        mouseoverDisplay = Tooltip.CreateExtendedToolTip(parent, text, body)
    else
        mouseoverDisplay = Tooltip.CreateToolTip(parent, text)
    end
    if extendedBool then
        local Frame = GetFrame(0)
        if parent.Top() - mouseoverDisplay.Height() < 0 then
            mouseoverDisplay.Top:Set(function() return parent.Bottom() + 10 end)
        else
            mouseoverDisplay.Bottom:Set(parent.Top)
        end
        if (parent.Left() + (parent.Width() / 2)) - (mouseoverDisplay.Width() / 2) < 0 then
            mouseoverDisplay.Left:Set(parent.Right)
        elseif (parent.Right() - (parent.Width() / 2)) + (mouseoverDisplay.Width() / 2) > Frame.Right() then
            mouseoverDisplay.Right:Set(parent.Left)
        else
            LayoutHelpers.AtHorizontalCenterIn(mouseoverDisplay, parent)
        end
    else
        local Frame = GetFrame(0)
        mouseoverDisplay.Bottom:Set(parent.Top)
        if (parent.Left() + (parent.Width() / 2)) - (mouseoverDisplay.Width() / 2) < 0 then
            mouseoverDisplay.Left:Set(4)
        elseif (parent.Right() - (parent.Width() / 2)) + (mouseoverDisplay.Width() / 2) > Frame.Right() then
            mouseoverDisplay.Right:Set(function() return Frame.Right() - 4 end)
        else
            LayoutHelpers.AtHorizontalCenterIn(mouseoverDisplay, parent)
        end
    end
    if ID == "mfd_defense" then
        local size = table.getn(mouseoverDisplay.desc)
        mouseoverDisplay.desc[size]:SetColor('ffff0000')
        mouseoverDisplay.desc[size-1]:SetColor('ffffff00')
        mouseoverDisplay.desc[size-2]:SetColor('ff00ff00')
        mouseoverDisplay.desc[size-3]:SetColor('ff4f77f4')
    end
    mouseoverDisplay:SetAlpha(alpha, true)
    mouseoverDisplay:SetNeedsFrameUpdate(true)
    mouseoverDisplay.OnFrame = function(self, deltaTime)
        if totalTime > createDelay then
            if parent then
                if alpha < 1 then
                    mouseoverDisplay:SetAlpha(alpha, true)
                    alpha = alpha + (deltaTime * 4)
                else
                    mouseoverDisplay:SetAlpha(1, true)
                    mouseoverDisplay:SetNeedsFrameUpdate(false)
                end
            else
                WARN( "MANIMAL\`s TooltipPlus: NO PARENT SPECIFIED FOR TOOLTIP" )
            end
        end
        totalTime = totalTime + deltaTime
    end
end



################################################################################
# Copie de la fonction originale assurant que MES fonctions seront appeles.

function DestroyMouseoverDisplay()
    if createThread then
        KillThread(createThread)
    end
    if mouseoverDisplay then
        mouseoverDisplay:Destroy()
        mouseoverDisplay = false
    end
end


################################################################################
# MODIFICATIONS :
# - AJOUT de OnMouseEnter() pour DIFFERENCIER les VNEMENTS SOURIS
# - tooltipID sera de type 'table' pour AFFICHER TooltipInfo DE statictitle
# - tooltipID sera de type 'string' pour AFFICHER TooltipInfo DE itemlist
################################################################################

function AddComboTooltip( control, tooltipID, tooltipTable, optPosition )
    local locParent = optPosition or control
	
    control.OnMouseExit = function( self )
        DestroyMouseoverDisplay()
    end
	
    control.OnMouseEnter = function( self ) # Uses the tooltipID
		CreateMouseoverDisplay( locParent, tooltipID, nil, true )--tooltip popup here
    end
	
    control.OnOverItem = function( self, index, text ) # Uses the tooltipTable
		--tooltip popup here, note, -1 is possible index (which means not over an item)
		if index ~= -1 and tooltipTable[index] then
			CreateMouseoverDisplay( locParent, tooltipTable[index], nil, true )
		else
			DestroyMouseoverDisplay()
		end
    end
end


################################################################################
# Copie de la fonction originale assurant que MES fonctions seront appeles.

function RemoveComboTooltip( control )
    control.OnMouseExit = function( self )
    end
    control.OnMouseEnter = function( self )
    end
    control.OnOverItem = function( self, index, text )
    end
end



################################################################################
# FONCTIONS Perso

--# NETTOYER LA LIGNE DE COMMANDE CONTENANT DES PARAMETRES
--# DO NOT KEEP PARAMETERS
function GetFilteredCommand( cmd )
	local commande = "" 
	local iFin = string.find( cmd, " ", 1 )
	if ( iFin ~= nil ) then
		commande = string.sub( cmd, 1, (iFin - 1) )
	else
		commande = cmd
	end
	for k, v in CommandsList do
		if v.commande == commande then
			return commande
		end
	end
    return cmd
end


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Crer un seul Tooltip  Partir d'une ligne de Commande
# Ncessite la Liste obtenue par GetCommandsListFromConsole()
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# ex: Renvoie TooltipID = "Info"  partir de commandLine = "Info My Batch RoX !" 
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

function GetCommandTooltipFromCommandLine( text, prefix )
    local tooltipTxt = ""
	if text then
		tooltipTxt = GetFilteredCommand( cmd )
		if prefix and ( type( prefix ) == 'string' ) and ( string.len( prefix ) > 0 ) then 
			tooltipTxt = prefix .. tooltipTxt     # Command Tooltip with specified prefix
		else
			tooltipTxt = 'cpcmd_' .. tooltipTxt   # Command Tooltip with predefined prefix
		end
	end
    return tooltipTxt
end


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Crer une Liste de Tooltips  Partir du Batch de Commandes
# Ncessite la Liste obtenue par GetCommandsListFromConsole()
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# mesCommandesConsole = { commande = "...", descrA = "...", info_aide = "...", }
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

function GetCommandTooltipListFromBatch( batch, prefix )
    local tooltipTable = {}
	if batch and ( type( batch ) == 'table' ) and ( table.getn( batch ) > 0 ) then
		for i, cmd in ipairs( batch ) do    # Copier toutes les commandes prsentes
			local tooltipTxt = GetFilteredCommand( cmd )
			if prefix and ( type( prefix ) == 'string' ) and ( string.len( prefix ) > 0 ) then 
				table.insert( tooltipTable, prefix .. tooltipTxt )    # Command Tooltip with specified prefix
			else
				table.insert( tooltipTable, 'cpcmd_' .. tooltipTxt )  # Command Tooltip with predefined prefix
			end
		end
	end
   return tooltipTable
end



#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Crer une Liste de Tooltips  Partir de la Sortie des Commandes
# Ncessite la Liste obtenue par GetCommandsListFromConsole()
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# mesCommandesConsole = { commande = "...", descrA = "...", info_aide = "...", }
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

function GetCommandTooltipListFromFuncsList( funcsList, prefix )
    local tooltipTable = {}
	if funcsList then
		for i = 0, ( funcsList:GetItemCount() - 1 ) do
			local cmd = funcsList:GetItem( i )  # Copier tous les items prsents
			local tooltipTxt = GetFilteredCommand( cmd )
			if prefix and ( type( prefix ) == 'string' ) and ( string.len( prefix ) > 0 ) then 
				table.insert( tooltipTable, prefix .. tooltipTxt )    # Command Tooltip with specified prefix
			else
				table.insert( tooltipTable, 'cpcmd_' .. tooltipTxt )  # Command Tooltip with predefined prefix
			end
		end
	end
    return tooltipTable
end



--[[################################################################################
# Cration des ToolTips Correspondant aux Commandes Originales de la Console et des Commandes Perso
#  partir de leur description et leur aide en ligne (info-aide).
# Ncessite la Liste obtenue par GetCommandsListFromConsole()
################################################################################ ]]--
--[[
EXEMPLE DE TOOLTIP
    consplus_help_close = {
        title = "<LOC tooltipuiCP021>Close Help",
        description = "<LOC tooltipuiCP022>Click here to close the Console Plus Help window.",
    },
EXEMPLE DE COMMANDSLIST
mesCommandesConsole = {
    {
		commande = "Info",
		descrA = LOC( "<LOC mCC_Info_desc>Displays some Text on Console Display. e.g. Info Hello Halcyon!" ),
		descrB = "",    # NEANT
		info_aide = "", # AUCUNE !
    },
    #etc
}
 ]]--

function CreateCommandTooltipsFromCommandsList()
	local cmdsTooltips = {}
	for k, cmd in pairs( CommandsList ) do  --# Sparer les Commandes et leur description. 
		cmdsTooltips['cpcmd_' .. cmd.commande] = { title = cmd.commande, description = cmd.descrA .. " " .. cmd.descrB, }
	end
	TooltipInfoPlus = table.merged( cmdsTooltips, TooltipInfoPlus )  # Dynamic Tool Tips Info :)
end



